import com.alibaba.fastjson.JSON;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.googlecode.aviator.Options;
import com.googlecode.aviator.exception.ExpressionSyntaxErrorException;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


@Slf4j
public class PointDataCalculator extends BasePointData {


    private static final Pattern PAR_EXPR = Pattern.compile("P\\(\\d+\\)");

    private static final Pattern RANDOM_EXPR = Pattern.compile("R\\((\\d+)?\\)");

    private static final Pattern TIME_EXPR = Pattern.compile("T\\(\\)");


    static {
        AviatorEvaluator.setOption(Options.ALWAYS_PARSE_FLOATING_POINT_NUMBER_INTO_DECIMAL, true);
    }

    /**
     * 计算表达式
     */
    Expression compiledExp;

    /**
     * 计算所需参数
     */
    private Map<String, Integer> calculatingParameters;

    public PointDataCalculator(Integer point, String formula) {
        super(point,formula);
        getOrInitCompiledExp();
    }

    public Object calculate() {
        Map<String, Object> values = new HashMap<>();
        for (Map.Entry<String, Integer> e : getCalculatingParameters().entrySet()) {
            values.put(e.getKey(), protocolDataContainer.getData(e.getValue()));
        }
        Object o = null;
        try {
            o=getOrInitCompiledExp().execute(values);
        } catch (ArithmeticException e) {
            log.error("除数为0,计算公式为：{},请检查数据池内是否有该点位是否有数据", getValue());
        } catch (Exception e) {
            log.error("数据计算{}时发生异常", getValue(), e);
            log.info(JSON.toJSONString(values));
            throw new RuntimeException("计算器计算数据时发生异常,公式为：" +getValue());
        }
        return o;
    }


    /**
     * 对公式进行初始化  如果公式解析失败将会抛出异常
     *
     * @return Expression  解析完成的表达式
     */
    private Expression getOrInitCompiledExp() {
        if (this.compiledExp == null) {
            try {
                String temS;
                String temS2;
                String temFormula = getValue();
                Matcher m = RANDOM_EXPR.matcher(getValue());
                while (m.find()) {
                    temS = m.group();
                    System.out.println(temS);
                    temS2 = temS.replace("R", "rand");
                    temFormula = temFormula.replace(temS, temS2);
                }
                m = TIME_EXPR.matcher(getValue());
                while (m.find()) {
                    temS = m.group();
                    temS2 = temS.replace("T", "now");
                    temFormula = temFormula.replace(temS, temS2);
                }
                System.out.println(temFormula);
                this.compiledExp = AviatorEvaluator.compile(temFormula);
            } catch (ExpressionSyntaxErrorException e) {
                log.error("公式解析失败，公式不可用", e);
                throw new RuntimeException("公式解析失败，公式不可用");
            }
        }
        return this.compiledExp;
    }

    /**
     * 对计算所需的参数进行整理
     *
     * @return P_（点位）_P  替换成真实的数据   字段--数值
     */
    public Map<String, Integer> getCalculatingParameters() {
        if (this.calculatingParameters == null) {
            this.calculatingParameters = new HashMap<>();
            Matcher matcher = PAR_EXPR.matcher(getValue());
            while (matcher.find()) {
                String s = matcher.group();
                s = s.substring(1, s.length() - 1);
                System.out.println(s);
                this.calculatingParameters.put("P_" + s + "_P", Integer.parseInt(s));
            }
        }
        return this.calculatingParameters;
    }


    @Override
    public void joinRealTimeData() {
        protocolDataContainer.addCalculator(this);
    }
}
